import 'dart:async';
import 'dart:io';

import 'package:flulm_auth/src/browser.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter/widgets.dart';

import '../flulm_auth.dart';

StreamController<LMAuthResponse> _responseAuthController =
    new StreamController.broadcast();

StreamController<LMAuthResponse> _responsePayController =
    new StreamController.broadcast();

/// Response from auth  自动登录
Stream<LMAuthResponse> get responseFromAuth => _responseAuthController.stream;

Stream<LMAuthResponse> get responseFromPay => _responsePayController.stream;

MethodChannel _channel = const MethodChannel('com.awesome/flulm_auth')
  ..setMethodCallHandler(_handler);

Future<dynamic> _handler(MethodCall methodCall) {
  if ("onAuthResponse" == methodCall.method) {
    _responseAuthController.add(
      LMAuthResponse.fromMap(methodCall.arguments),
    );
  } else if ("onPayResponse" == methodCall.method) {
    _responsePayController.add(
      LMAuthResponse.fromMap(methodCall.arguments),
    );
  }
  return Future.value(true);
}

Future<void> sendAuth(
    {String openId, @required String scope, String state}) async {
  assert(scope != null && scope.trim().isNotEmpty);
  return await _channel.invokeMethod("sendAuth", {"scope": scope});
}

Future<bool> isLemaPayInstalled() =>
    _channel.invokeMethod("isLemaPayInstalled").then((install) {
      return install;
    });

Future<int> isLemaPaySupportAPI() =>
    _channel.invokeMethod("isLemaPaySupportAPI").then((version) {
      return version;
    });

///调用乐马支付
Future<void> sendPay(
    {@required BuildContext mContext,
    @required String openId,
    @required String appKey,
    @required String prepayId,
    @required String coOrderId,
    @required String payH5Url,
    @required String packageValue,
    @required String nonceStr,
    @required String timeStamp}) async {
  assert(mContext != null &&
      openId != null &&
      openId.trim().isNotEmpty &&
      appKey != null &&
      appKey.trim().isNotEmpty &&
      prepayId != null &&
      prepayId.trim().isNotEmpty &&
      coOrderId != null &&
      coOrderId.trim().isNotEmpty &&
      payH5Url != null &&
      payH5Url.trim().isNotEmpty &&
      packageValue != null &&
      packageValue.trim().isNotEmpty &&
      nonceStr != null &&
      nonceStr.trim().isNotEmpty &&
      timeStamp != null &&
      timeStamp.trim().isNotEmpty);

  //如果是IOS苹果或者没有安装乐马调用h5支付
  bool isInstalledLema = false;
  if (Platform.isAndroid) {
    isInstalledLema = await isLemaPayInstalled();
  }
  if (Platform.isIOS || !isInstalledLema) {
    await Navigator.push(mContext,
        MaterialPageRoute(builder: (context) => Browser(url: payH5Url)));
    Map map = Map();
    map["errCode"] = 999;
    _responsePayController.add(
      LMAuthResponse.fromMap(map),
    );
    return;
  }

  return await _channel.invokeMethod("sendPay", {
    "openId": openId,
    "appKey": appKey,
    "prepayId": prepayId,
    "coOrderId": coOrderId,
    "pay_h5_url": payH5Url,
    "packageValue": packageValue,
    "nonceStr": nonceStr,
    "timeStamp": timeStamp,
  });
}

///[appId] is not necessary.
///if [doOnIOS] is true ,fluwx will register WXApi on iOS.
///if [doOnAndroid] is true, fluwx will register WXApi on Android.
Future register(
    {@required String appId,
    bool doOnIOS: true,
    doOnAndroid: true,
    enableMTA: false}) async {
  return await _channel.invokeMethod("registerApp", {
    "appId": appId,
    "iOS": doOnIOS,
    "android": doOnAndroid,
    "enableMTA": enableMTA
  });
}
